#ifndef DUCHAMP_FINDER_H_
#define DUCHAMP_FINDER_H_

#include <iostream>
#include <duchamp/duchamp.hh>
#include <duchamp/param.hh>
#include <duchamp/fitsHeader.hh>
#include <duchamp/DataArrays/DataArrayBase.hh>
#include <duchamp/DataArrays/AstroImage.hh>
#include <duchamp/Catalogues/Catalogue.hh>
#include <duchamp/Detection/detection.hh>

namespace duchamp
{

    class DuchampFinder
    {
    public:
	DuchampFinder(){};
	DuchampFinder(const DuchampFinder& other){};
	DuchampFinder& operator= (const DuchampFinder& other){};
	virtual ~DuchampFinder(){};
	
	void readParams();
	void readHeader();
	void readData();
	
	void preprocess();
	void invert();
	void reInvert();
	void smooth();
	void atrousRecon();

	void subtractBaselines();
	void replaceBaselines();
	
	void readFromCatalogue();
	
	FitsHeader *header(){return &itsHeader;};
	Param *par(){return &itsParam;};
	Statistics::StatsContainer<float> *stats(){return &itsStats;};
	AstroImage<float> *image(){return &itsImage;};
	AstroImage<float> *recon(){return &itsRecon;};
	AstroImage<float> *baseline(){return &itsBaselines;};
	AstroImage<PIXELSTATUS> *mask(){return &itsSmartMask;};
	AstroImage<short> *detectmap(){return &itsDetectMap;};
      

    protected:
	FitsHeader itsHeader;
	Param      itsParam;
	Statistics::StatsContainer<float> itsStats;
	AstroImage<float> itsImage;
	AstroImage<float> itsRecon;
	AstroImage<float> itsBaselines;
	AstroImage<PIXELSTATUS> itsSmartMask;
	AstroImage<short> itsDetectMap;

	Catalogue<Detection> itsCatalogue;
    };
    
}

#endif

